@@ -76,6 +76,7 @@ except ImportError:
 
 DEFAULTS = {
         'global_config':   {
+            'dbus'                  : False,
             'focus'                 : 'click',
             'handle_size'           : -1,
             'geometry_hinting'      : True,
@@ -85,14 +86,21 @@ DEFAULTS = {
             'close_button_on_tab'   : True,
             'hide_tabbar'           : False,
             'scroll_tabbar'         : False,
+            'hide_from_taskbar'     : False,
+            'always_on_top'         : False,
+            'hide_on_lose_focus'    : False,
+            'sticky'                : False,
             'try_posix_regexp'      : platform.system() != 'Linux',
+            'title_hide_sizetext'   : False,
             'title_transmit_fg_color' : '#ffffff',
             'title_transmit_bg_color' : '#c80003',
             'title_receive_fg_color' : '#ffffff',
             'title_receive_bg_color' : '#0076c9',
             'title_inactive_fg_color' : '#000000',
             'title_inactive_bg_color' : '#c0bebf',
-            'disabled_plugins'      : ['TestPlugin', 'CustomCommandsMenu'],
+            'enabled_plugins'       : ['LaunchpadBugURLHandler',
+                                       'LaunchpadCodeURLHandler',
+                                       'APTURLHandler'],
         },
         'keybindings': {
             'zoom_in'          : '<Control>plus',
@@ -145,10 +153,14 @@ DEFAULTS = {
             'ungroup_tab'      : '<Shift><Super>t',
             'new_window'       : '<Shift><Control>i',
             'new_terminator'   : '<Super>i',
+            'broadcast_off'    : '',
+            'broadcast_group'  : '',
+            'broadcast_all'    : ''
         },
         'profiles': {
             'default':  {
                 'allow_bold'            : True,
+                'antialias'             : True,
                 'audible_bell'          : False,
                 'visible_bell'          : False,
                 'urgent_bell'           : False,
@@ -225,7 +237,7 @@ class Config(object):
     
     def __init__(self, profile='default'):
         self.base = ConfigBase()
-        self.profile = profile
+        self.set_profile(profile)
         self.inhibited = False
 
     def __getitem__(self, key):
@@ -240,8 +252,12 @@ class Config(object):
         """Get our profile"""
         return(self.profile)
 
-    def set_profile(self, profile):
+    def set_profile(self, profile, force=False):
         """Set our profile (which usually means change it)"""
+        options = self.options_get()
+        if not force and options and options.profile and profile == 'default':
+            dbg('overriding default profile to %s' % options.profile)
+            profile = options.profile
         dbg('Config::set_profile: Changing profile to %s' % profile)
         self.profile = profile
         if not self.base.profiles.has_key(profile):
@@ -255,10 +271,16 @@ class Config(object):
     def del_profile(self, profile):
         """Delete a profile"""
         if profile == self.profile:
+            # FIXME: We should solve this problem by updating terminals when we
+            # remove a profile
             err('Config::del_profile: Deleting in-use profile %s.' % profile)
             self.set_profile('default')
         if self.base.profiles.has_key(profile):
             del(self.base.profiles[profile])
+        options = self.options_get()
+        if options and options.profile == profile:
+            options.profile = None
+            self.options_set(options)
 
     def rename_profile(self, profile, newname):
         """Rename a profile"""
@@ -496,6 +518,7 @@ class ConfigBase(Borg):
             return
 
         filename = os.path.join(get_config_dir(), 'config')
+        dbg('looking for config file: %s' % filename)
         try:
             configfile = open(filename, 'r')
         except Exception, ex:
@@ -596,6 +619,10 @@ class ConfigBase(Borg):
 
     def get_item(self, key, profile='default', plugin=None):
         """Look up a configuration item"""
+        if not self.profiles.has_key(profile):
+            # Hitting this generally implies a bug
+            profile = 'default'
+
         if self.global_config.has_key(key):
             dbg('ConfigBase::get_item: %s found in globals: %s' %
                     (key, self.global_config[key]))
