@@ -24,6 +24,7 @@ from util import dbg, err
 import util
 import config
 import version
+from translation import _
 
 def execute_cb(option, opt, value, lparser):
     """Callback for use in parsing execute options"""
@@ -43,37 +44,41 @@ def parse_options():
     parser = OptionParser(usage)
 
     parser.add_option('-v', '--version', action='store_true', dest='version',
-            help='Display program version')
+            help=_('Display program version'))
     parser.add_option('-m', '--maximise', action='store_true', dest='maximise',
-            help='Maximise the window')
+            help=_('Maximise the window'))
     parser.add_option('-f', '--fullscreen', action='store_true',
-            dest='fullscreen', help='Make the window fill the screen')
+            dest='fullscreen', help=_('Make the window fill the screen'))
     parser.add_option('-b', '--borderless', action='store_true',
-            dest='borderless', help='Disable window borders')
+            dest='borderless', help=_('Disable window borders'))
     parser.add_option('-H', '--hidden', action='store_true', dest='hidden',
-            help='Hide the window at startup')
-    parser.add_option('-T', '--title', dest='forcedtitle', help='Specify a \
-title for the window')
-    parser.add_option('--geometry', dest='geometry', type='string', help='Set \
-the preferred size and position of the window (see X man page)')
-    parser.add_option('-e', '--command', dest='command', help='Specify a \
-command to execute inside the terminal')
+            help=_('Hide the window at startup'))
+    parser.add_option('-T', '--title', dest='forcedtitle', help=_('Specify a \
+title for the window'))
+    parser.add_option('--geometry', dest='geometry', type='string', help=_('Set \
+the preferred size and position of the window (see X man page)'))
+    parser.add_option('-e', '--command', dest='command', help=_('Specify a \
+command to execute inside the terminal'))
     parser.add_option('-x', '--execute', dest='execute', action='callback',
-            callback=execute_cb, help='Use the rest of the command line as a \
-command to execute inside the terminal, and its arguments')
+            callback=execute_cb, help=_('Use the rest of the command line as a \
+command to execute inside the terminal, and its arguments'))
     parser.add_option('--working-directory', metavar='DIR',
-            dest='working_directory', help='Set the working directory')
-    parser.add_option('-r', '--role', dest='role', help='Set a custom \
-WM_WINDOW_ROLE property on the window')
-    parser.add_option('-l', '--layout', dest='layout', help='Select a layout')
+            dest='working_directory', help=_('Set the working directory'))
+    parser.add_option('-r', '--role', dest='role', help=_('Set a custom \
+WM_WINDOW_ROLE property on the window'))
+    parser.add_option('-l', '--layout', dest='layout', help=_('Select a layout'))
+    parser.add_option('-p', '--profile', dest='profile', help=_('Use a \
+different profile as the default'))
+    parser.add_option('-u', '--no-dbus', action='store_true', dest='nodbus', 
+            help=_('Disable DBus'))
     parser.add_option('-d', '--debug', action='count', dest='debug',
-            help='Enable debugging information (twice for debug server)')
+            help=_('Enable debugging information (twice for debug server)'))
     parser.add_option('--debug-classes', action='store', dest='debug_classes', 
-            help='Comma separated list of classes to limit debugging to')
+            help=_('Comma separated list of classes to limit debugging to'))
     parser.add_option('--debug-methods', action='store', dest='debug_methods',
-            help='Comma separated list of methods to limit debugging to')
+            help=_('Comma separated list of methods to limit debugging to'))
     for item in ['--sm-client-id', '--sm-config-prefix', '--screen', '-n',
-    '--no-gconf', '-p', '--profile' ]:
+    '--no-gconf' ]:
         parser.add_option(item, dest='dummy', action='store',
                 help=SUPPRESS_HELP)
 
@@ -113,6 +118,9 @@ WM_WINDOW_ROLE property on the window')
     if options.layout is None:
         options.layout = 'default'
 
+    if options.profile and options.profile not in configobj.list_profiles():
+        options.profile = None
+
     configobj.options_set(options)
 
     if util.DEBUG == True:
