@@ -4,11 +4,11 @@
 """window.py - class for the main Terminator window"""
 
 import copy
+import time
 import pygtk
 pygtk.require('2.0')
 import gobject
 import gtk
-import glib
 
 from util import dbg, err
 import util
@@ -34,9 +34,20 @@ class Window(Container, gtk.Window):
     ismaximised = None
     hidebound = None
     hidefunc = None
+    losefocus_time = 0
+    position = None
+    ignore_startup_show = None
 
     zoom_data = None
-    term_zoomed = gobject.property(type=bool, default=False)
+
+    term_zoomed = False
+    __gproperties__ = {
+            'term_zoomed': (gobject.TYPE_BOOLEAN,
+                            'terminal zoomed',
+                            'whether the terminal is zoomed',
+                            False,
+                            gobject.PARAM_READWRITE)
+    }
 
     def __init__(self):
         """Class initialiser"""
@@ -70,12 +81,31 @@ class Window(Container, gtk.Window):
                     err('Window::__init__: Unable to parse geometry: %s' % 
                             options.geometry)
 
+        self.pending_set_rough_geometry_hint = False
+
+    def do_get_property(self, prop):
+        """Handle gobject getting a property"""
+        if prop.name in ['term_zoomed', 'term-zoomed']:
+            return(self.term_zoomed)
+        else:
+            raise AttributeError('unknown property %s' % prop.name)
+
+    def do_set_property(self, prop, value):
+        """Handle gobject setting a property"""
+        if prop.name in ['term_zoomed', 'term-zoomed']:
+            self.term_zoomed = value
+        else:
+            raise AttributeError('unknown property %s' % prop.name)
+
     def register_callbacks(self):
         """Connect the GTK+ signals we care about"""
         self.connect('key-press-event', self.on_key_press)
+        self.connect('button-press-event', self.on_button_press)
         self.connect('delete_event', self.on_delete_event)
         self.connect('destroy', self.on_destroy_event)
         self.connect('window-state-event', self.on_window_state_changed)
+        self.connect('focus-out-event', self.on_focus_out)
+        self.connect('focus-in-event', self.on_focus_in)
 
         # Attempt to grab a global hotkey for hiding the window.
         # If we fail, we'll never hide the window, iconifying instead.
@@ -99,6 +129,9 @@ class Window(Container, gtk.Window):
         fullscreen = self.config['window_state'] == 'fullscreen'
         hidden = self.config['window_state'] == 'hidden'
         borderless = self.config['borderless']
+        skiptaskbar = self.config['hide_from_taskbar']
+        alwaysontop = self.config['always_on_top']
+        sticky = self.config['sticky']
 
         if options:
             if options.maximise:
@@ -113,9 +146,12 @@ class Window(Container, gtk.Window):
         self.set_fullscreen(fullscreen)
         self.set_maximised(maximise)
         self.set_borderless(borderless)
+        self.set_always_on_top(alwaysontop)
         self.set_real_transparency()
+        self.set_sticky(sticky)
         if self.hidebound:
             self.set_hidden(hidden)
+            self.set_skip_taskbar_hint(skiptaskbar)
         else:
             self.set_iconified(hidden)
 
@@ -125,7 +161,7 @@ class Window(Container, gtk.Window):
 
         try:
             icon = icon_theme.load_icon(APP_NAME, 48, 0)
-        except (NameError, glib.GError):
+        except (NameError, gobject.GError):
             dbg('Unable to load 48px Terminator icon')
             icon = self.render_icon(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_BUTTON)
 
@@ -154,6 +190,27 @@ class Window(Container, gtk.Window):
                 return(False)
             return(True)
 
+    def on_button_press(self, window, event):
+        """Handle a mouse button event. Mainly this is just a clean way to
+        cancel any urgency hints that are set."""
+        self.set_urgency_hint(False)
+        return(False)
+
+    def on_focus_out(self, window, event):
+        """Focus has left the window"""
+        for terminal in self.get_visible_terminals():
+            terminal.on_window_focus_out()
+
+        self.losefocus_time = time.time()
+        if self.config['hide_on_lose_focus'] and self.get_property('visible'):
+            self.position = self.get_position()
+            self.hidefunc()
+
+    def on_focus_in(self, window, event):
+        """Focus has entered the window"""
+        self.set_urgency_hint(False)
+        # FIXME: Cause the terminal titlebars to update here
+
     def is_child_notebook(self):
         """Returns True if this Window's child is a Notebook"""
         maker = Factory()
@@ -171,6 +228,7 @@ class Window(Container, gtk.Window):
             cwd = widget.get_cwd()
         maker = Factory()
         if not self.is_child_notebook():
+            dbg('Making a new Notebook')
             notebook = maker.make('Notebook', window=self)
         self.get_child().newtab(debugtab, cwd=cwd)
 
@@ -194,8 +252,10 @@ class Window(Container, gtk.Window):
         return(not (result == gtk.RESPONSE_ACCEPT))
 
     def on_destroy_event(self, widget, data=None):
-        """Handle window descruction"""
+        """Handle window destruction"""
         dbg('destroying self')
+        for terminal in self.get_visible_terminals():
+            terminal.close()
         self.cnxids.remove_all()
         self.terminator.deregister_window(self)
         self.destroy()
@@ -203,8 +263,19 @@ class Window(Container, gtk.Window):
 
     def on_hide_window(self, data=None):
         """Handle a request to hide/show the window"""
-        # FIXME: Implement or drop, or explain why its empty
-        pass
+
+        if not self.get_property('visible'):
+            #Don't show if window has just been hidden because of
+            #lost focus
+            if (time.time() - self.losefocus_time < 0.1) and \
+                self.config['hide_on_lose_focus']:
+                return
+            if self.position:
+                self.move(self.position[0], self.position[1])
+            self.show()
+        else:
+            self.position = self.get_position()
+            self.hidefunc()
 
     # pylint: disable-msg=W0613
     def on_window_state_changed(self, window, event):
@@ -213,8 +284,8 @@ class Window(Container, gtk.Window):
                                  gtk.gdk.WINDOW_STATE_FULLSCREEN)
         self.ismaximised = bool(event.new_window_state &
                                  gtk.gdk.WINDOW_STATE_MAXIMIZED)
-        dbg('Window::on_window_state_changed: fullscreen=%s, maximised=%s' %
-            (self.isfullscreen, self.ismaximised))
+        dbg('Window::on_window_state_changed: fullscreen=%s, maximised=%s' \
+                % (self.isfullscreen, self.ismaximised))
 
         return(False)
 
@@ -238,13 +309,24 @@ class Window(Container, gtk.Window):
 
     def set_hidden(self, value):
         """Set the visibility of the window from the supplied value"""
-        # FIXME: Implement or drop this
-        pass
+        if value == True:
+            self.ignore_startup_show = True
+        else:
+            self.ignore_startup_show = False
 
     def set_iconified(self, value):
-        """Set the minimised state of the window from the value"""
-        # FIXME: Implement or drop this
-        pass
+        """Set the minimised state of the window from the supplied value"""
+        if value == True:
+            self.iconify()
+
+    def set_always_on_top(self, value):
+        """Set the always on top window hint from the supplied value"""
+        self.set_keep_above(value)
+
+    def set_sticky(self, value):
+        """Set the sticky hint from the supplied value"""
+        if value == True:
+            self.stick()
 
     def set_real_transparency(self, value=True):
         """Enable RGBA if supported on the current screen"""
@@ -261,8 +343,19 @@ class Window(Container, gtk.Window):
 
         if colormap:
             self.set_colormap(colormap)
+    
+    def show(self, startup=False):
+        """Undo the startup show request if started in hidden mode"""
+        gtk.Window.show(self)
+        #Present is necessary to grab focus when window is hidden from taskbar
+        self.present()
 
-    def add(self, widget):
+        #Window must be shown, then hidden for the hotkeys to be registered
+        if (self.ignore_startup_show and startup == True):
+            self.hide()
+
+
+    def add(self, widget, metadata=None):
         """Add a widget to the window by way of gtk.Window.add()"""
         maker = Factory()
         gtk.Window.add(self, widget)
@@ -389,9 +482,14 @@ class Window(Container, gtk.Window):
     def get_visible_terminals(self):
         """Walk down the widget tree to find all of the visible terminals.
         Mostly using Container::get_visible_terminals()"""
-        maker = Factory()
-        child = self.get_child()
         terminals = {}
+        if not hasattr(self, 'cached_maker'):
+            self.cached_maker = Factory()
+        maker = self.cached_maker
+        child = self.get_child()
+
+        if not child:
+            return([])
 
         # If our child is a Notebook, reset to work from its visible child
         if maker.isinstance(child, 'Notebook'):
@@ -415,10 +513,24 @@ class Window(Container, gtk.Window):
                 return(terminal)
         return(None)
 
+    def deferred_set_rough_geometry_hints(self):
+        # no parameters are used in set_rough_geometry_hints, so we can
+        # use the set_rough_geometry_hints
+        if self.pending_set_rough_geometry_hint == True:
+            return
+        self.pending_set_rough_geometry_hint = True
+        gobject.idle_add(self.do_deferred_set_rough_geometry_hints)
+
+    def do_deferred_set_rough_geometry_hints(self):
+        self.pending_set_rough_geometry_hint = False
+        self.set_rough_geometry_hints()
+
     def set_rough_geometry_hints(self):
         """Walk all the terminals along the top and left edges to fake up how
         many columns/rows we sort of have"""
-        maker = Factory()
+        if not hasattr(self, 'cached_maker'):
+            self.cached_maker = Factory()
+        maker = self.cached_maker
         if maker.isinstance(self.get_child(), 'Notebook'):
             dbg("We don't currently support geometry hinting with tabs")
             return
@@ -623,9 +735,10 @@ class Window(Container, gtk.Window):
 
             if len(winners) > 1:
                 # Break an n-way tie using the cursor position
-                cursor_x, cursor_y = terminal.get_cursor_position()
-                cursor_x = cursor_x + allocation.x
-                cursor_y = cursor_y + allocation.y
+                term_alloc = terminal.allocation
+                cursor_x = term_alloc.x + term_alloc.width / 2
+                cursor_y = term_alloc.y + term_alloc.height / 2
+
                 for term in winners:
                     rect = layout[term]
                     if util.get_nav_tiebreak(direction, cursor_x, cursor_y,
@@ -651,12 +764,17 @@ class Window(Container, gtk.Window):
 
         child = children[children.keys()[0]]
         terminal = self.get_children()[0]
+        dbg('Making a child of type: %s' % child['type'])
         if child['type'] == 'VPaned':
             self.split_axis(terminal, True)
         elif child['type'] == 'HPaned':
             self.split_axis(terminal, False)
         elif child['type'] == 'Notebook':
             self.tab_new()
+            i = 2
+            while i < len(child['children']):
+                self.tab_new()
+                i = i + 1
         elif child['type'] == 'Terminal':
             pass
         else:
